'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createReducerHandlers = exports.createFields = exports.getEntities = exports.denormalize = undefined;

var _reduxObject = require('redux-object');

var _reduxObject2 = _interopRequireDefault(_reduxObject);

var _reselect = require('reselect');

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _map = require('lodash/map');

var _map2 = _interopRequireDefault(_map);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isEmpty = require('lodash/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _entityHandlers = require('./entityHandlers');

var _helpers = require('./helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var denormalize = exports.denormalize = function denormalize(entities, type, id) {
  return (0, _reduxObject2.default)(entities, type, (0, _isArray2.default)(id) ? (0, _map2.default)(id, function (i) {
    return i.id || i;
  }) : id, {
    eager: true
  });
};

var getEntities = exports.getEntities = function getEntities(getState, getData, _ref) {
  var type = _ref.type,
      field = _ref.field,
      _ref$singular = _ref.singular,
      singular = _ref$singular === undefined ? false : _ref$singular;
  return (0, _reselect.createSelector)(getState, getData, function (state, data) {
    var entity = field || type;

    var addKey = !field || field === type ? '' : (0, _helpers.capitalizeFirstLetter)(field);

    var all = {
      isLoading: state['isLoading' + addKey],
      isLoaded: state['isLoaded' + addKey]
    };

    var ids = state[entity];

    var entityKey = singular ? 'entity' : 'entities';

    if (!(0, _isEmpty2.default)(ids)) {
      all[entityKey] = denormalize(data, type, ids);
    } else {
      all[entityKey] = singular ? {} : [];
    }

    return all;
  });
};

var createFields = exports.createFields = function createFields(type, field) {
  var _ref2;

  var singular = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  var entity = field || type;
  var addKey = !field || field === type ? '' : (0, _helpers.capitalizeFirstLetter)(field);

  return _ref2 = {}, _defineProperty(_ref2, entity, singular ? null : []), _defineProperty(_ref2, 'isLoading' + addKey, false), _defineProperty(_ref2, 'isLoaded' + addKey, false), _ref2;
};

var createReducerHandlers = exports.createReducerHandlers = function createReducerHandlers(type, actionTypes) {
  var _ref3;

  var handlerOptions = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  var field = (0, _get2.default)(handlerOptions, 'mapToKey');
  var addKey = !field || field === type ? '' : (0, _helpers.capitalizeFirstLetter)(field);

  return _ref3 = {}, _defineProperty(_ref3, actionTypes.REQUEST, function (state) {
    var _state$merge;

    return state.merge((_state$merge = {}, _defineProperty(_state$merge, 'isLoading' + addKey, true), _defineProperty(_state$merge, 'isLoaded' + addKey, false), _state$merge));
  }), _defineProperty(_ref3, actionTypes.SUCCESS, (0, _entityHandlers.createLoadHandler)(type, handlerOptions)), _defineProperty(_ref3, actionTypes.FAILURE, function (state) {
    var _state$merge2;

    return state.merge((_state$merge2 = {}, _defineProperty(_state$merge2, 'isLoading' + addKey, false), _defineProperty(_state$merge2, 'isLoaded' + addKey, false), _state$merge2));
  }), _ref3;
};