'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createRelationDeleteHandler = exports.createRelationAddHandler = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _merge = require('lodash/merge');

var _merge2 = _interopRequireDefault(_merge);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _mergeWith2 = require('lodash/mergeWith');

var _mergeWith3 = _interopRequireDefault(_mergeWith2);

var _reject = require('lodash/reject');

var _reject2 = _interopRequireDefault(_reject);

var _mergers = require('./mergers');

var _mergers2 = _interopRequireDefault(_mergers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var getRelationId = function getRelationId(relationName, value) {
  return (0, _get2.default)(value, 'relationships.' + relationName + '.data.id');
};

var extractRelationsIds = function extractRelationsIds(relationName) {
  return function (relationsIds, current) {
    var relationId = getRelationId(relationName, current);
    var hasRelId = relationsIds.indexOf(relationId) >= 0;
    return hasRelId ? relationsIds : [].concat(_toConsumableArray(relationsIds), [relationId]);
  };
};

var createRelationAddHandler = exports.createRelationAddHandler = function createRelationAddHandler(type, relationName, options) {
  return function (state, _ref) {
    var payload = _ref.payload;

    var dataType = (0, _get2.default)(payload, 'data.' + type);

    if (!dataType) return state;

    var createdRelations = Object.values(dataType);
    var updatedRelativesMap = new Map();

    var mapToKey = (0, _get2.default)(options, 'mapToKey') || type;
    var singular = (0, _get2.default)(options, 'singular') === true;

    createdRelations.reduce(extractRelationsIds(relationName), []) // collect affected relatives ids
    .forEach(function (relationId) {
      return updatedRelativesMap.set(relationId, state[relationId]);
    }); // collect affected relatives state

    createdRelations.forEach(function (current) {
      var currentRelationId = getRelationId(relationName, current);
      if (!currentRelationId) return;
      var relative = updatedRelativesMap.get(currentRelationId);
      var newRelation = { id: current.id, type: type };
      var existedRelations = (0, _get2.default)(relative, 'relationships.' + type + '.data', {});
      var updatedRelative = _extends({}, relative, {
        relationships: _defineProperty({}, mapToKey, {
          data: singular ? newRelation : [].concat(_toConsumableArray(existedRelations), [newRelation])
        })
      });
      updatedRelativesMap.set(currentRelationId, updatedRelative); // Override prev value with prev+new
    });

    // Convert Map<RelationId, RelationValue> to Array<Relation>
    var updatedRelatives = [];
    // eslint-disable-next-line no-restricted-syntax
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = updatedRelativesMap.entries()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var _step$value = _slicedToArray(_step.value, 2),
            key = _step$value[0],
            value = _step$value[1];

        if (key && value) {
          updatedRelatives.push(_defineProperty({}, key, value));
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return _merge2.default.apply(undefined, [{}, state].concat(updatedRelatives));
  };
};

var createRelationDeleteHandler = exports.createRelationDeleteHandler = function createRelationDeleteHandler(relationsName) {
  return function (state, _ref2) {
    var payload = _ref2.payload;

    var deletedId = (0, _get2.default)(payload, 'deletedId');
    var relationId = (0, _get2.default)(payload, 'relationId');

    if (!deletedId || !relationId) return state;

    return (0, _mergeWith3.default)({}, state, _defineProperty({}, relationId, {
      relationships: _defineProperty({}, relationsName, {
        data: (0, _reject2.default)(state[relationId].relationships[relationsName].data, {
          id: deletedId
        })
      })
    }), _mergers2.default.latestArrayMerger);
  };
};